#!/usr/bin/env python

import os
import unittest

import PIL

from diff import diff

IMAGES_DIR = "images"


def mk_img_path(f):
    return os.path.join(IMAGES_DIR, f)


IMG1 = mk_img_path("mario-circle-cs.png")
IMG2 = mk_img_path("mario-circle-node.png")
IMG3 = mk_img_path("yandex1.png")  # RGBA
IMG4 = mk_img_path("yandex2.png")  # RGB
TEST_JPG_OUT = mk_img_path("test-diff.jpg")
TEST_PNG_OUT = mk_img_path("test-diff.png")
TEST_NO_EXT_OUT = mk_img_path("test-diff")
WHITE_IMG = mk_img_path("white.png")
BLACK_IMG = mk_img_path("black.png")


class TestAll(unittest.TestCase):
    def test_mario_ratio(self):
        ratio = diff(IMG1, IMG2, delete_diff_file=True)
        self.assertEqual(ratio, 0.007319618135968298)

    def test_bw_image_ratio(self):
        ratio = diff(BLACK_IMG, WHITE_IMG, delete_diff_file=True)
        self.assertEqual(ratio, 1.0)

    def test_delete_diff_img(self):
        if os.path.exists(TEST_JPG_OUT):
            os.remove(TEST_JPG_OUT)
        diff(IMG1, IMG2, delete_diff_file=True, diff_img_file=TEST_JPG_OUT)
        self.assertFalse(os.path.exists(TEST_JPG_OUT))

    def test_make_jpg_diff_img(self):
        diff(IMG1, IMG2, diff_img_file=TEST_JPG_OUT)
        self.assertTrue(os.path.exists(TEST_JPG_OUT))
        os.remove(TEST_JPG_OUT)

    def test_different_modes(self):
        with self.assertRaises(ValueError):
            diff(IMG3, IMG4)

    def test_make_png_diff_img(self):
        # these images are the same, but it shouldn't matter for this test
        diff(IMG4, IMG4, diff_img_file=TEST_PNG_OUT)
        self.assertTrue(os.path.exists(TEST_PNG_OUT))
        os.remove(TEST_PNG_OUT)

    def test_diff_filename_without_extension_saves_as_png(self):
        diff(IMG4, IMG4, diff_img_file=TEST_NO_EXT_OUT)
        im = PIL.Image.open(TEST_NO_EXT_OUT)
        self.assertEqual(im.format, "PNG")
        os.remove(TEST_NO_EXT_OUT)

    def test_ignore_alpha_mario(self):
        ratio = diff(IMG1, IMG2, delete_diff_file=True, ignore_alpha=True)
        self.assertEqual(ratio, 0.008675405566134298)

    def test_ignore_alpha_bw(self):
        ratio = diff(BLACK_IMG, WHITE_IMG, delete_diff_file=True, ignore_alpha=True)
        self.assertEqual(ratio, 1.0)

    def test_ignore_alpha_mario_file(self):
        diff(IMG1, IMG2, diff_img_file=TEST_PNG_OUT, ignore_alpha=True)
        im = PIL.Image.open(TEST_PNG_OUT)
        self.assertEqual(im.getextrema()[3], (255, 255))

    def test_different_image_sizes(self):
        ratio = diff(IMG1, IMG3, delete_diff_file=True, ignore_alpha=True)
        # this is the ratio for the (manually inspected) resulting diff image
        self.assertEqual(ratio, 0.3503192421617232)


if __name__ == "__main__":
    unittest.main()



"""
Return the % difference of two given images.
Only works with images of the same file type and color channels.
"""

from __future__ import print_function
from PIL import Image, ImageChops, ImageStat

DIFF_IMG_FILE = "diff_img.png"


def diff(
    im1_file, im2_file, delete_diff_file=False, diff_img_file=None, ignore_alpha=False
):
    """
    Calculate the difference between two images by comparing channel values at the pixel
    level. If the images are different sizes, the second will be resized to match the
    first.

    `delete_diff_file`: removes the diff image after ratio found
    `diff_img_file`: filename to store diff image
    `ignore_alpha`: ignore the alpha channel for ratio calculation, and set the diff
        image's alpha to fully opaque
    """
    if not diff_img_file:
        diff_img_file = DIFF_IMG_FILE

    im1 = Image.open(im1_file)
    im2 = Image.open(im2_file)

    # Ensure we have the same color channels (RGBA vs RGB)
    if im1.mode != im2.mode:
        raise ValueError(
            (
                "Differing color modes:\n  {}: {}\n  {}: {}\n"
                "Ensure image color modes are the same."
            ).format(im1_file, im1.mode, im2_file, im2.mode)
        )

    # Coerce 2nd dimensions to same as 1st
    im2 = im2.resize((im1.width, im1.height))

    # Generate diff image in memory.
    diff_img = ImageChops.difference(im1, im2)

    if ignore_alpha:
        diff_img.putalpha(256)

    if not delete_diff_file:
        if "." not in diff_img_file:
            extension = "png"
        else:
            extension = diff_img_file.split(".")[-1]
        if extension in ("jpg", "jpeg"):
            # For some reason, save() thinks "jpg" is invalid
            # This doesn't affect the image's saved filename
            extension = "jpeg"
            diff_img = diff_img.convert("RGB")
        diff_img.save(diff_img_file, extension)

    # Calculate difference as a ratio.
    stat = ImageStat.Stat(diff_img)
    # stat.mean can be [r,g,b] or [r,g,b,a].
    removed_channels = 1 if ignore_alpha and len(stat.mean) == 4 else 0
    num_channels = len(stat.mean) - removed_channels
    sum_channel_values = sum(stat.mean[:num_channels])
    max_all_channels = num_channels * 255
    diff_ratio = sum_channel_values / max_all_channels

    return diff_ratio



#!/usr/bin/env python

from __future__ import print_function
import argparse
import sys
from .diff import diff


def get_args():
    parser = argparse.ArgumentParser(
        prog="diffimg",
        description="Generate a diff image from two images \
            and/or find difference percentage",
    )
    parser.add_argument("image1", type=str, help="first image")
    parser.add_argument("image2", type=str, help="second image")
    parser.add_argument(
        "--ratio",
        "-r",
        dest="use_ratio",
        action="store_true",
        help="return a ratio instead of percentage",
    )
    parser.add_argument(
        "--delete",
        "-d",
        dest="delete_diff_file",
        action="store_true",
        help="delete diff image file",
    )
    parser.add_argument(
        "--filename",
        "-f",
        dest="diff_img_file",
        type=str,
        default=None,
        help="filename with valid extension to store diff image \
                (defaults to diff_img.jpg)",
    )
    parser.add_argument(
        "--ignore-alpha",
        "-ia",
        dest="ignore_alpha",
        action="store_true",
        help="ignore the alpha channel for ratio calculation \
                and diff img creation",
    )
    return parser.parse_args()


def main():
    args = get_args()
    diff_ratio = diff(
        args.image1,
        args.image2,
        delete_diff_file=args.delete_diff_file,
        diff_img_file=args.diff_img_file,
        ignore_alpha=args.ignore_alpha,
    )
    if args.use_ratio:
        print(diff_ratio)
    else:
        print("Images differ by {}%".format(diff_ratio * 100))
    return 0


if __name__ == "__main__":
    sys.exit(main())
